@@ -1,7 +1,3 @@
-# Copyright (c) 2001, 2002, 2003 Python Software Foundation
-# Copyright (c) 2004 Paramjit Oberoi <param.cs.wisc.edu>
-# All Rights Reserved.  See LICENSE-PSF & LICENSE for details.
-
 """Access and/or modify INI files
 
 * Compatiable with ConfigParser
@@ -42,10 +38,10 @@ Example:
 # Backward-compatiable with ConfigParser
 
 import re
-import config
-from sets import Set
 from ConfigParser import DEFAULTSECT, ParsingError, MissingSectionHeaderError
 
+import config
+
 class LineType(object):
     line = None
 
@@ -105,7 +101,7 @@ class SectionLine(LineType):
 
 
 class OptionLine(LineType):
-    def __init__(self, name, value, separator='=', comment=None,
+    def __init__(self, name, value, separator=' = ', comment=None,
                  comment_separator=None, comment_offset=-1, line=None):
         super(OptionLine, self).__init__(line)
         self.name = name
@@ -116,7 +112,7 @@ class OptionLine(LineType):
         self.comment_offset = comment_offset
 
     def to_string(self):
-        out = '%s %s %s' % (self.name, self.separator, self.value)
+        out = '%s%s%s' % (self.name, self.separator, self.value)
         if self.comment is not None:
             # try to preserve indentation of comments
             out = (out+' ').ljust(self.comment_offset)
@@ -124,7 +120,7 @@ class OptionLine(LineType):
         return out
 
     regex = re.compile(r'^(?P<name>[^:=\s[][^:=]*)'
-                       r'\s*(?P<sep>[:=])\s*'
+                       r'(?P<sep>[:=]\s*)'
                        r'(?P<value>.*)$')
 
     def parse(cls, line):
@@ -134,7 +130,7 @@ class OptionLine(LineType):
 
         name = m.group('name').rstrip()
         value = m.group('value')
-        sep = m.group('sep')
+        sep = m.group('name')[len(name):] + m.group('sep')
 
         # comments are not detected in the regex because
         # ensuring total compatibility with ConfigParser
@@ -187,6 +183,8 @@ class EmptyLine(LineType):
     def to_string(self):
         return ''
 
+    value = property(lambda _: '')
+
     def parse(cls, line):
         if line.strip(): return None
         return cls(line)
@@ -196,9 +194,11 @@ class EmptyLine(LineType):
 class ContinuationLine(LineType):
     regex = re.compile(r'^\s+(?P<value>.*)$')
 
-    def __init__(self, value, value_offset=8, line=None):
+    def __init__(self, value, value_offset=None, line=None):
         super(ContinuationLine, self).__init__(line)
         self.value = value
+        if value_offset is None:
+            value_offset = 8
         self.value_offset = value_offset
 
     def to_string(self):
@@ -238,25 +238,35 @@ class LineContainer(object):
         elif len(self.contents) == 1:
             return self.contents[0].value
         else:
-            return '\n'.join([str(x.value) for x in self.contents])
+            return '\n'.join([('%s' % x.value) for x in self.contents
+                              if not isinstance(x, CommentLine)])
 
     def set_value(self, data):
         self.orgvalue = data
-        lines = str(data).split('\n')
-        linediff = len(lines) - len(self.contents)
-        if linediff > 0:
-            for _ in range(linediff):
-                self.add(ContinuationLine(''))
-        elif linediff < 0:
-            self.contents = self.contents[:linediff]
-        for i,v in enumerate(lines):
-            self.contents[i].value = v
+        lines = ('%s' % data).split('\n')
+
+        # If there is an existing ContinuationLine, use its offset
+        value_offset = None
+        for v in self.contents:
+            if isinstance(v, ContinuationLine):
+                value_offset = v.value_offset
+                break
+
+        # Rebuild contents list, preserving initial OptionLine
+        self.contents = self.contents[0:1]
+        self.contents[0].value = lines[0]
+        del lines[0]
+        for line in lines:
+            if line.strip():
+                self.add(ContinuationLine(line, value_offset))
+            else:
+                self.add(EmptyLine())
 
     name = property(get_name, set_name)
     value = property(get_value, set_value)
 
     def __str__(self):
-        s = [str(x) for x in self.contents]
+        s = [x.__str__() for x in self.contents]
         return '\n'.join(s)
 
     def finditer(self, key):
@@ -269,27 +279,27 @@ class LineContainer(object):
             return x
         raise KeyError(key)
 
-        
+
 def _make_xform_property(myattrname, srcattrname=None):
     private_attrname = myattrname + 'value'
     private_srcname = myattrname + 'source'
     if srcattrname is None:
         srcattrname = myattrname
-    
+
     def getfn(self):
         srcobj = getattr(self, private_srcname)
         if srcobj is not None:
             return getattr(srcobj, srcattrname)
         else:
             return getattr(self, private_attrname)
-            
+
     def setfn(self, value):
         srcobj = getattr(self, private_srcname)
         if srcobj is not None:
             setattr(srcobj, srcattrname, value)
         else:
             setattr(self, private_attrname, value)
-            
+
     return property(getfn, setfn)
 
 
@@ -299,6 +309,7 @@ class INISection(config.ConfigNamespace):
     _defaults = None
     _optionxformvalue = None
     _optionxformsource = None
+    _compat_skip_empty_lines = set()
     def __init__(self, lineobj, defaults = None,
                        optionxformvalue=None, optionxformsource=None):
         self._lines = [lineobj]
@@ -308,7 +319,26 @@ class INISection(config.ConfigNamespace):
         self._options = {}
 
     _optionxform = _make_xform_property('_optionxform')
-    
+
+    def _compat_get(self, key):
+        # identical to __getitem__ except that _compat_XXX
+        # is checked for backward-compatible handling
+        if key == '__name__':
+            return self._lines[-1].name
+        if self._optionxform: key = self._optionxform(key)
+        try:
+            value = self._options[key].value
+            del_empty = key in self._compat_skip_empty_lines
+        except KeyError:
+            if self._defaults and key in self._defaults._options:
+                value = self._defaults._options[key].value
+                del_empty = key in self._defaults._compat_skip_empty_lines
+            else:
+                raise
+        if del_empty:
+            value = re.sub('\n+', '\n', value)
+        return value
+
     def __getitem__(self, key):
         if key == '__name__':
             return self._lines[-1].name
@@ -324,6 +354,8 @@ class INISection(config.ConfigNamespace):
     def __setitem__(self, key, value):
         if self._optionxform: xkey = self._optionxform(key)
         else: xkey = key
+        if xkey in self._compat_skip_empty_lines:
+            self._compat_skip_empty_lines.remove(xkey)
         if xkey not in self._options:
             # create a dummy object - value may have multiple lines
             obj = LineContainer(OptionLine(key, ''))
@@ -335,6 +367,8 @@ class INISection(config.ConfigNamespace):
 
     def __delitem__(self, key):
         if self._optionxform: key = self._optionxform(key)
+        if key in self._compat_skip_empty_lines:
+            self._compat_skip_empty_lines.remove(key)
         for l in self._lines:
             remaining = []
             for o in l.contents:
@@ -348,7 +382,7 @@ class INISection(config.ConfigNamespace):
         del self._options[key]
 
     def __iter__(self):
-        d = Set()
+        d = set()
         for l in self._lines:
             for x in l.contents:
                 if isinstance(x, LineContainer):
@@ -365,7 +399,7 @@ class INISection(config.ConfigNamespace):
                     yield x
                     d.add(x)
 
-    def new_namespace(self, name):
+    def _new_namespace(self, name):
         raise Exception('No sub-sections allowed', name)
 
 
@@ -373,6 +407,30 @@ def make_comment(line):
     return CommentLine(line.rstrip())
 
 
+def readline_iterator(f):
+    """iterate over a file by only using the file object's readline method"""
+
+    have_newline = False
+    while True:
+        line = f.readline()
+
+        if not line:
+            if have_newline:
+                yield ""
+            return
+
+        if line.endswith('\n'):
+            have_newline = True
+        else:
+            have_newline = False
+
+        yield line
+
+
+def lower(x):
+    return x.lower()
+
+
 class INIConfig(config.ConfigNamespace):
     _data = None
     _sections = None
@@ -382,8 +440,9 @@ class INIConfig(config.ConfigNamespace):
     _sectionxformvalue = None
     _sectionxformsource = None
     _parse_exc = None
-    def __init__(self, fp=None, defaults = None, parse_exc=True,
-                 optionxformvalue=str.lower, optionxformsource=None,
+    _bom = False
+    def __init__(self, fp=None, defaults=None, parse_exc=True,
+                 optionxformvalue=lower, optionxformsource=None,
                  sectionxformvalue=None, sectionxformsource=None):
         self._data = LineContainer()
         self._parse_exc = parse_exc
@@ -397,11 +456,11 @@ class INIConfig(config.ConfigNamespace):
         for name, value in defaults.iteritems():
             self._defaults[name] = value
         if fp is not None:
-            self.readfp(fp)
+            self._readfp(fp)
 
     _optionxform = _make_xform_property('_optionxform', 'optionxform')
     _sectionxform = _make_xform_property('_sectionxform', 'optionxform')
-        
+
     def __getitem__(self, key):
         if key == DEFAULTSECT:
             return self._defaults
@@ -418,14 +477,15 @@ class INIConfig(config.ConfigNamespace):
         del self._sections[key]
 
     def __iter__(self):
-        d = Set()
+        d = set()
+        d.add(DEFAULTSECT)
         for x in self._data.contents:
             if isinstance(x, LineContainer):
                 if x.name not in d:
                     yield x.name
                     d.add(x.name)
 
-    def new_namespace(self, name):
+    def _new_namespace(self, name):
         if self._data.contents:
             self._data.add(EmptyLine())
         obj = LineContainer(SectionLine(name))
@@ -441,7 +501,13 @@ class INIConfig(config.ConfigNamespace):
         return ns
 
     def __str__(self):
-        return str(self._data)
+        if self._bom:
+            fmt = u'\ufeff%s'
+        else:
+            fmt = '%s'
+        return fmt % self._data.__str__()
+
+    __unicode__ = __str__
 
     _line_types = [EmptyLine, CommentLine,
                    SectionLine, OptionLine,
@@ -456,19 +522,28 @@ class INIConfig(config.ConfigNamespace):
             # can't parse line
             return None
 
-    def readfp(self, fp):
+    def _readfp(self, fp):
         cur_section = None
         cur_option = None
         cur_section_name = None
         cur_option_name = None
         pending_lines = []
+        pending_empty_lines = False
         try:
             fname = fp.name
         except AttributeError:
             fname = '<???>'
         linecount = 0
         exc = None
-        for line in fp:
+        line = None
+
+        for line in readline_iterator(fp):
+            # Check for BOM on first line
+            if linecount == 0 and isinstance(line, unicode):
+                if line[0] == u'\ufeff':
+                    line = line[1:]
+                    self._bom = True
+
             lineobj = self._parse(line)
             linecount += 1
 
@@ -487,6 +562,12 @@ class INIConfig(config.ConfigNamespace):
 
             if isinstance(lineobj, ContinuationLine):
                 if cur_option:
+                    if pending_lines:
+                        cur_option.extend(pending_lines)
+                        pending_lines = []
+                        if pending_empty_lines:
+                            optobj._compat_skip_empty_lines.add(cur_option_name)
+                            pending_empty_lines = False
                     cur_option.add(lineobj)
                 else:
                     # illegal continuation line - convert to comment
@@ -494,13 +575,14 @@ class INIConfig(config.ConfigNamespace):
                         if exc is None: exc = ParsingError(fname)
                         exc.append(linecount, line)
                     lineobj = make_comment(line)
-            else:
-                cur_option = None
-                cur_option_name = None
 
             if isinstance(lineobj, OptionLine):
-                cur_section.extend(pending_lines)
-                pending_lines = []
+                if pending_lines:
+                    cur_option.extend(pending_lines)
+                    pending_lines = []
+                    if pending_empty_lines:
+                        optobj._compat_skip_empty_lines.add(cur_option_name)
+                        pending_empty_lines = False
                 cur_option = LineContainer(lineobj)
                 cur_section.add(cur_option)
                 if self._optionxform:
@@ -516,8 +598,11 @@ class INIConfig(config.ConfigNamespace):
             if isinstance(lineobj, SectionLine):
                 self._data.extend(pending_lines)
                 pending_lines = []
+                pending_empty_lines = False
                 cur_section = LineContainer(lineobj)
                 self._data.add(cur_section)
+                cur_option = None
+                cur_option_name = None
                 if cur_section.name == DEFAULTSECT:
                     self._defaults._lines.append(cur_section)
                     cur_section_name = DEFAULTSECT
@@ -526,7 +611,7 @@ class INIConfig(config.ConfigNamespace):
                         cur_section_name = self._sectionxform(cur_section.name)
                     else:
                         cur_section_name = cur_section.name
-                    if not self._sections.has_key(cur_section_name):
+                    if cur_section_name not in self._sections:
                         self._sections[cur_section_name] = \
                                 INISection(cur_section, defaults=self._defaults,
                                            optionxformsource=self)
@@ -535,6 +620,8 @@ class INIConfig(config.ConfigNamespace):
 
             if isinstance(lineobj, (CommentLine, EmptyLine)):
                 pending_lines.append(lineobj)
+                if isinstance(lineobj, EmptyLine):
+                    pending_empty_lines = True
 
         self._data.extend(pending_lines)
         if line and line[-1]=='\n':
