@@ -1,12 +1,9 @@
-# Copyright (c) 2001, 2002, 2003 Python Software Foundation
-# Copyright (c) 2004 Paramjit Oberoi <param.cs.wisc.edu>
-# All Rights Reserved.  See LICENSE-PSF & LICENSE for details.
-
 import unittest, doctest
 
 import test_ini
 import test_misc
 import test_compat
+import test_unicode
 from iniparse import config
 from iniparse import ini
 
@@ -18,4 +15,5 @@ class suite(unittest.TestSuite):
                 test_ini.suite(),
                 test_misc.suite(),
                 test_compat.suite(),
+                test_unicode.suite(),
         ])
